@@ -1,3 +1,4 @@
+import json
 import logging
 
 # set a logger for debug, log format is:
@@ -15,4 +16,12 @@ DebugEnable = False
 
 def Debug(msg, *args):
     if DebugEnable:
+        if isinstance(msg, dict):
+            msg = json.dumps(msg, indent=2, ensure_ascii=False)
         _log.debug(msg, *args)
+
+
+def Warning(msg, *args):
+    if isinstance(msg, dict):
+        msg = json.dumps(msg, indent=2, ensure_ascii=False)
+    _log.warning(msg, *args)
