@@ -3,7 +3,7 @@ from enum import IntEnum
 
 class ErrorCode(IntEnum):
     SUCCESS = 0
-    UNEXPECTED_ERROR = 1
+    UNEXPECTED_ERROR = -1
 
 
 class VectorDBException(Exception):
@@ -37,11 +37,19 @@ class ConnectError(VectorDBException):
 
 
 class ServerInternalError(VectorDBException):
-    """database core errorr"""
+    """database core error"""
 
 
 class IndexTypeException(VectorDBException):
     """Raise when one field is invalid"""
 
+
 class DescribeCollectionException(VectorDBException):
     """Raise when describe collection error"""
+
+
+class AIUnsupportedException(VectorDBException):
+    """Raise when method unsupported"""
+
+class GrpcException(VectorDBException):
+    """Raise when grpc exception"""
