@@ -1,20 +1,42 @@
-import json
-from typing import Dict, List, Optional, Any
+from typing import Dict, List, Optional, Any, Union
+
+from tcvectordb.model.index import IndexField, VectorIndex, FilterIndex
 
 from tcvectordb import exceptions
-from .document import Document, Filter
+from .document import Document, Filter, AnnSearch, KeywordSearch, Rerank
 from .enum import EmbeddingModel, ReadConsistency
 from .index import Index
 
 
 class Embedding:
+    """init Embedding"""
+
     def __init__(self, vector_field: str = None, status: str = 'disabled', field: str = None,
-                 model: EmbeddingModel = None):
+                 model: EmbeddingModel = None, model_name: str = None):
+        """
+        init Embedding when create embedding collection
+
+        Args:
+            vector_field (str): vector field name
+            status (str): status of embedding, enable is available, disabled is unavailable, status is invalid when
+                create collection
+            field (str): field name of embedding content
+            model (EmbeddingModel): [Deprecated] embedding model enum, this is a deprecated parameter, it will be
+                instead of arg model_name, if model and model_name both are not None, model_name is used first.
+            model_name (str): model_name specify embedding model wher upsert documents
+
+        Examples:
+            >>> Embedding(vector_field="vector", field="text", model=EmbeddingModel.BGE_BASE_ZH)
+            >>> Embedding(vector_field="vector", field="text", model_name="bge-large-zh")
+        """
         self._status = status
 
-        if field is not None and model is not None and vector_field is not None:
+        if field is not None and vector_field is not None:
             self._field = field
-            self._model = model.model_name
+            if model_name is not None:
+                self._model = model_name
+            elif model is not None:
+                self._model = model.model_name
             self._vector_field = vector_field
 
     @property
@@ -88,8 +110,8 @@ class Query(BaseQuery):
     """
 
     def __init__(self,
-                 limit: int = 10,
-                 offset: int = 0,
+                 limit: Optional[int] = None,
+                 offset: Optional[int] = None,
                  retrieve_vector: bool = False,
                  filter: Optional[Filter] = None,
                  document_ids: Optional[List] = None,
@@ -97,7 +119,6 @@ class Query(BaseQuery):
                  ):
 
         super().__init__(filter, document_ids)
-
         self._limit = limit
         self._offset = offset
         self._retrieve_vector = retrieve_vector
@@ -109,13 +130,13 @@ class Query(BaseQuery):
     def __dict__(self):
         res = {
             "retrieveVector": self._retrieve_vector,
-            "limit": self._limit,
-            "offset": self._offset,
         }
-
+        if self._limit is not None:
+            res["limit"] = self._limit
+        if self._offset is not None:
+            res["offset"] = self._offset
         if hasattr(self, "_output_fields"):
             res["outputFields"] = self._output_fields
-
         res.update(super().__dict__)
         return res
 
@@ -155,15 +176,11 @@ class Search:
                  ):
         self._retrieve_vector = retrieve_vector
         self._limit = limit
-
-        if vectors is not None:
-            self._vectors = vectors
-
+        self.vectors = vectors
         if document_ids is not None:
             self._document_ids = document_ids
 
-        if embedding_items is not None:
-            self._embedding_items = embedding_items
+        self.embedding_items = embedding_items
 
         if params is not None:
             self._params = params
@@ -181,14 +198,14 @@ class Search:
             "limit": self._limit,
         }
 
-        if hasattr(self, "_vectors"):
-            res["vectors"] = self._vectors
+        if self.vectors is not None:
+            res["vectors"] = self.vectors
 
         if hasattr(self, "_document_ids"):
             res["documentIds"] = self._document_ids
 
-        if hasattr(self, "_embedding_items"):
-            res["embeddingItems"] = self._embedding_items
+        if self.embedding_items is not None:
+            res["embeddingItems"] = self.embedding_items
 
         if hasattr(self, "_params"):
             res["params"] = vars(self._params)
@@ -236,9 +253,12 @@ class Collection():
         self.description = description
         self._embedding = embedding
         self._index = index
-        self._coll_info = {}
-        self.create_time = kwargs.get('create_time', '')
+        self.create_time = kwargs.pop('createTime', None)
+        self.document_count = kwargs.pop("documentCount", None)
+        self.alias = kwargs.pop("alias", None)
+        self.index_status = kwargs.pop("indexStatus", None)
         self._read_consistency = read_consistency
+        self.kwargs = kwargs
 
     @property
     def database_name(self):
@@ -252,24 +272,14 @@ class Collection():
     def index(self):
         return self._index
 
+    @property
+    def indexes(self) -> List[Union[IndexField, VectorIndex, FilterIndex]]:
+        return list(self.index.indexes.values())
+
     @property
     def embedding(self):
         return self._embedding
 
-    def set_fields(self, **kwargs):
-        coll_info = {}
-
-        if "documentCount" in kwargs:
-            coll_info["documentCount"] = kwargs.get("documentCount")
-
-        if "alias" in kwargs:
-            coll_info["alias"] = kwargs.get("alias")
-
-        if "indexStatus" in kwargs:
-            coll_info["indexStatus"] = kwargs.get("indexStatus")
-
-        self._coll_info = coll_info
-
     @property
     def __dict__(self):
         res_dict = {
@@ -277,21 +287,28 @@ class Collection():
             'collection': self.collection_name,
             'replicaNum': self.replicas,
             'shardNum': self.shard,
-            'description': self.description,
             'indexes': self.index.list(),
-            'embedding': vars(self._embedding) if self._embedding is not None else {}
         }
-
-        if len(self._coll_info) > 0:
-            res_dict.update(self._coll_info)
-
+        if self._embedding is not None:
+            res_dict['embedding'] = vars(self._embedding)
+        if self.description:
+            res_dict['description'] = self.description
+        if self.create_time is not None:
+            res_dict['createTime'] = self.create_time
+        if self.document_count is not None:
+            res_dict['documentCount'] = self.document_count
+        if self.alias is not None:
+            res_dict['alias'] = self.alias
+        if self.index_status is not None:
+            res_dict['indexStatus'] = self.index_status
         return res_dict
 
     def upsert(
             self,
-            documents: List[Document],
+            documents: List[Union[Document, Dict]],
             timeout: Optional[float] = None,
             build_index: bool = True,
+            **kwargs
     ):
         """Upsert a document.
 
@@ -299,30 +316,47 @@ class Collection():
         :type  documents: list[Document]
 
         :param build_index: An option for build index time when upsert, if build_index is true, will build index
-                            immediately, it will affect performance of upsert.
+                            immediately, it will affect performance of upsert. And param buildIndex has same
+                            semantics with build_index, any of them false will be false
         :type  build_index: bool
 
         :param timeout: An optional duration of time in seconds to allow for the request. When timeout
                         is set to None, will use the connect timeout.
         :type  timeout: float
+
+        :param buildIndex:  An option for build index time when upsert, if build_index is buildIndex, will build index
+                            immediately, it will affect performance of upsert. And param build_index has same
+                            semantics with build_index, any of them false will be false
+        :type buildIndex: bool
         """
+        buildIndex = bool(kwargs.get("buildIndex", True))
+        res_build_index = buildIndex and build_index
         body = {
             'database': self.database_name,
             'collection': self.collection_name,
-            'buildIndex': build_index,
+            'buildIndex': res_build_index,
             'documents': []
         }
+        ai = False
+        if len(documents) > 0:
+            if isinstance(documents[0], dict):
+                ai = isinstance(documents[0].get('vector'), str)
+            else:
+                ai = isinstance(vars(documents[0]).get('vector'), str)
         for doc in documents:
-            body['documents'].append(vars(doc))
-
-        self._conn.post('/document/upsert', body, timeout)
+            if isinstance(doc, dict):
+                body['documents'].append(doc)
+            else:
+                body['documents'].append(vars(doc))
+        res = self._conn.post('/document/upsert', body, timeout, ai=ai)
+        return res.data()
 
     def query(
             self,
             document_ids: Optional[List] = None,
             retrieve_vector: bool = False,
-            limit: Optional[int] = 1,
-            offset: Optional[int] = 0,
+            limit: Optional[int] = None,
+            offset: Optional[int] = None,
             filter: Optional[Filter] = None,
             output_fields: Optional[List[str]] = None,
             timeout: Optional[float] = None,
@@ -345,10 +379,6 @@ class Collection():
         :return Documents, the list of the document
         :rtype: list[Dict]
         """
-        if not isinstance(document_ids, list):
-            raise exceptions.ParamError(
-                code=-1, message='document_ids parameter must be set as list')
-
         query_param = Query(limit=limit, offset=offset, retrieve_vector=retrieve_vector, filter=filter,
                             document_ids=document_ids, output_fields=output_fields)
         return self.__base_query(query=query_param, read_consistency=self._read_consistency, timeout=timeout)
@@ -517,7 +547,11 @@ class Collection():
             'readConsistency': read_consistency.value,
             'search': vars(search)
         }
-        res = self._conn.post('/document/search', body, timeout)
+        ai = False
+        if isinstance(search.vectors, list) and \
+                len(search.vectors) > 0 and isinstance(search.vectors[0], str):
+            ai = True
+        res = self._conn.post('/document/search', body, timeout, ai=ai)
 
         warn_msg = ""
         if res.body.get("warning", None) is not None and len(res.body.get("warning", None)) > 0:
@@ -543,11 +577,100 @@ class Collection():
             'documents': documents_res
         }
 
+    def hybrid_search(self,
+                      ann: Optional[List[AnnSearch]] = None,
+                      match: Optional[List[KeywordSearch]] = None,
+                      filter: Optional[Filter] = None,
+                      rerank: Optional[Rerank] = None,
+                      retrieve_vector: Optional[bool] = None,
+                      output_fields: Optional[List[str]] = None,
+                      limit: Optional[int] = None,
+                      timeout: Optional[float] = None,
+                      **kwargs) -> List[List[Dict]]:
+        """hybrid search
+
+        :param match: ann params for search.
+        :type match : List[KeywordSearch]
+
+        :param ann: sparse vector search params.
+        :type ann : List[AnnSearch]
+
+        :param filter: The optional filter condition of the scalar index field.
+        :type filter: Filter
+
+        :param rerank: rerank params for search.
+        :type rerank : Rerank
+
+        :param output_fields: document's fields to return.
+        :type output_fields: list[str]
+
+        :param retrieve_vector: Whether to return vector values.
+        :type retrieve_vector: bool
+
+        :param limit: the limit of the query result, not support now
+        :type: int
+
+        :param timeout: An optional duration of time in seconds to allow for the request. When timeout
+                        is set to None, will use the connect timeout.
+        :type  timeout: float
+
+        :return Documents, the list of the document
+        :rtype: List[List[Dict]]
+        """
+        search = {}
+        ai = False
+        if ann:
+            search['ann'] = []
+            for a in ann:
+                search['ann'].append(vars(a))
+            if len(ann) > 0 and ann[0].data is not None:
+                if isinstance(ann[0].data, str):
+                    ai = True
+                elif len(ann[0].data) > 0 and isinstance(ann[0].data[0], str):
+                    ai = True
+        if match:
+            search['match'] = []
+            for m in match:
+                search['match'].append(vars(m))
+        if filter:
+            search['filter'] = vars(filter)
+        if rerank:
+            # if rerank.method == "wordsEmbedding":
+            #     ai = True
+            search['rerank'] = vars(rerank)
+        if retrieve_vector is not None:
+            search['retrieveVector'] = retrieve_vector
+        if output_fields:
+            search['outputFields'] = output_fields
+        if limit is not None:
+            search['limit'] = limit
+        search.update(kwargs)
+        body = {
+            'database': self.database_name,
+            'collection': self.collection_name,
+            'readConsistency': self._read_consistency.value,
+            'search': search,
+        }
+        res = self._conn.post('/document/hybridSearch', body, timeout, ai=ai)
+        if 'warning' in res.body:
+            Warning(res.body.get('warning'))
+        documents = res.body.get('documents', None)
+        if not documents:
+            return []
+        documents_res = []
+        for arr in documents:
+            tmp = []
+            for elem in arr:
+                tmp.append(elem)
+            if tmp:
+                documents_res.append(tmp)
+        return documents_res
+
     def delete(
             self,
-            document_ids: Optional[List[str]] = None,
-            filter: Optional[Filter] = None,
-            timeout: Optional[float] = None,
+            document_ids: List[str] = None,
+            filter: Filter = None,
+            timeout: float = None,
     ):
         """Delete document by document id list.
 
@@ -558,12 +681,9 @@ class Collection():
                         is set to None, will use the connect timeout.
         :type  timeout: float
         """
-        if not isinstance(document_ids, list):
-            raise exceptions.ParamError(
-                code=-1, message='document_ids parameter must be set as list')
 
         delete_query_param = DeleteQuery(document_ids=document_ids, filter=filter)
-        self.__base_delete(delete_query=delete_query_param, timeout=timeout)
+        return self.__base_delete(delete_query=delete_query_param, timeout=timeout)
 
     def __base_delete(
             self,
@@ -583,29 +703,33 @@ class Collection():
         if not self.database_name or not self.collection_name:
             raise exceptions.ParamError(message="database_name or collection_name is blank")
 
-        if delete_query is not None and not delete_query.valid():
-            raise exceptions.ParamError(code=-1,
-                                        message='base_query both field document_ids and filter are None')
+        # if delete_query is not None and not delete_query.valid():
+        #     raise exceptions.ParamError(code=-1,
+        #                                 message='base_query both field document_ids and filter are None')
 
         body = {
             "database": self.database_name,
             "collection": self.collection_name,
             "query": vars(delete_query)
         }
-        self._conn.post('/document/delete', body, timeout)
+        res = self._conn.post('/document/delete', body, timeout)
+        return res.data()
 
-    def update(self, data: Document, filter: Optional[Filter] = None, document_ids: Optional[List[str]] = None,
+    def update(self,
+               data: Union[Document, Dict],
+               filter: Optional[Filter] = None,
+               document_ids: Optional[List[str]] = None,
                timeout: Optional[float] = None):
-        if filter is None and document_ids is None:
-            raise exceptions.ParamError(code=-1, message='both field document_ids and filter are None')
-
         if data is None:
             raise exceptions.ParamError(code=-1, message='data is None')
 
         update_query = UpdateQuery(document_ids=document_ids, filter=filter)
         return self.__base_update(update_query=update_query, document=data, timeout=timeout)
 
-    def __base_update(self, update_query: UpdateQuery, document: Document, timeout: Optional[float] = None) -> Dict:
+    def __base_update(self,
+                      update_query: UpdateQuery,
+                      document: Union[Document, Dict],
+                      timeout: Optional[float] = None) -> Dict:
         if not self.database_name or not self.collection_name:
             raise exceptions.ParamError(message="database_name or collection_name is blank")
 
@@ -614,23 +738,18 @@ class Collection():
 
         if document is None:
             raise exceptions.ParamError(code=-1, message='document is None')
-
         body = {
             'database': self.database_name,
             'collection': self.collection_name,
             'query': vars(update_query)
         }
-
-        doc_dict = vars(document)
-
-        if "vector" in doc_dict and len(doc_dict.get("vector")) > 0:
-            pop = doc_dict.pop("vector")
-            vector = [x for x in pop if type(x) == float]
-            if len(vector) > 0:
-                body["vector"] = vector
-
-        body["update"] = doc_dict
-        postRes = self._conn.post('/document/update', body, timeout)
+        ai = False
+        if isinstance(document, dict):
+            ai = isinstance(document.get('vector'), str)
+        else:
+            ai = isinstance(vars(document).get('vector'), str)
+        body["update"] = document if isinstance(document, dict) else vars(document)
+        postRes = self._conn.post('/document/update', body, timeout, ai=ai)
         resBody = postRes.body
         res = {}
 
