@@ -9,9 +9,12 @@ class FieldType(Enum):
     # scalar field type
     Uint64 = "uint64"
     String = "string"
+    Array = "array"
 
     # vector field type
     Vector = "vector"
+    # sparse vector type
+    SparseVector = "sparseVector"
 
 
 @unique
@@ -38,6 +41,7 @@ class IndexType(Enum):
     # scalar index type
     PRIMARY_KEY = "primaryKey"
     FILTER = "filter"
+    SPARSE_INVERTED = "inverted"
 
 
 @unique
@@ -68,5 +72,3 @@ class ReadConsistency(Enum):
     @property
     def value(self):
         return self.__value
-
-
